@@ -1,25 +1,28 @@
+use core::marker::PhantomData;
+
 use crate::{
-    api::{EndpointArgumentApi, ManagedTypeApi},
+    api::{EndpointArgumentApi, EndpointArgumentApiImpl, ErrorApi, ErrorApiImpl, ManagedTypeApi},
     err_msg, ArgDecodeInput, DynArgInput,
 };
 
+#[derive(Default)]
 pub struct EndpointDynArgLoader<AA>
 where
-    AA: ManagedTypeApi + EndpointArgumentApi,
+    AA: ManagedTypeApi + ErrorApi + EndpointArgumentApi,
 {
-    api: AA,
+    _phantom: PhantomData<AA>,
     current_index: i32,
     num_arguments: i32,
 }
 
 impl<AA> EndpointDynArgLoader<AA>
 where
-    AA: ManagedTypeApi + EndpointArgumentApi,
+    AA: ManagedTypeApi + ErrorApi + EndpointArgumentApi,
 {
-    pub fn new(api: AA) -> Self {
-        let num_arguments = api.get_num_arguments();
+    pub fn new() -> Self {
+        let num_arguments = AA::argument_api_impl().get_num_arguments();
         EndpointDynArgLoader {
-            api,
+            _phantom: PhantomData,
             current_index: 0,
             num_arguments,
         }
@@ -28,16 +31,11 @@ where
 
 impl<AA> DynArgInput for EndpointDynArgLoader<AA>
 where
-    AA: ManagedTypeApi + EndpointArgumentApi,
+    AA: ManagedTypeApi + ErrorApi + EndpointArgumentApi,
 {
     type ItemInput = ArgDecodeInput<AA>;
 
-    type ErrorApi = AA;
-
-    #[inline]
-    fn dyn_arg_vm_api(&self) -> Self::ErrorApi {
-        self.api.clone()
-    }
+    type ManagedTypeErrorApi = AA;
 
     fn has_next(&self) -> bool {
         self.current_index < self.num_arguments
@@ -45,10 +43,9 @@ where
 
     fn next_arg_input(&mut self) -> ArgDecodeInput<AA> {
         if self.current_index >= self.num_arguments {
-            self.dyn_arg_vm_api()
-                .signal_error(err_msg::ARG_WRONG_NUMBER)
+            AA::error_api_impl().signal_error(err_msg::ARG_WRONG_NUMBER)
         } else {
-            let arg_input = ArgDecodeInput::new(self.api.clone(), self.current_index);
+            let arg_input = ArgDecodeInput::new(self.current_index);
             self.current_index += 1;
             arg_input
         }
