@@ -1,7 +1,9 @@
 use crate::{
-    api::{ErrorApi, ManagedTypeApi, StorageReadApi},
+    api::{ErrorApi, ErrorApiImpl, ManagedTypeApi, StorageReadApi, StorageReadApiImpl},
     err_msg,
-    types::{BigInt, BigUint, ManagedBuffer, ManagedBufferNestedDecodeInput, ManagedType},
+    types::{
+        BigInt, BigUint, ManagedBuffer, ManagedBufferNestedDecodeInput, ManagedRef, ManagedType,
+    },
 };
 use alloc::boxed::Box;
 use elrond_codec::*;
@@ -12,8 +14,7 @@ struct StorageGetInput<'k, A>
 where
     A: StorageReadApi + ManagedTypeApi + ErrorApi + 'static,
 {
-    api: A,
-    key: &'k StorageKey<A>,
+    key: ManagedRef<'k, A, StorageKey<A>>,
 }
 
 impl<'k, A> StorageGetInput<'k, A>
@@ -21,13 +22,12 @@ where
     A: StorageReadApi + ManagedTypeApi + ErrorApi + 'static,
 {
     #[inline]
-    fn new(api: A, key: &'k StorageKey<A>) -> Self {
-        StorageGetInput { api, key }
+    fn new(key: ManagedRef<'k, A, StorageKey<A>>) -> Self {
+        StorageGetInput { key }
     }
 
     fn to_managed_buffer(&self) -> ManagedBuffer<A> {
-        let mbuf_handle = self
-            .api
+        let mbuf_handle = A::storage_read_api_impl()
             .storage_load_managed_buffer_raw(self.key.buffer.get_raw_handle());
         ManagedBuffer::from_raw_handle(mbuf_handle)
     }
@@ -41,8 +41,7 @@ where
     }
 
     fn load_len_managed_buffer(&self) -> usize {
-        self.api
-            .storage_load_managed_buffer_len(self.key.buffer.get_raw_handle())
+        A::storage_read_api_impl().storage_load_managed_buffer_len(self.key.buffer.get_raw_handle())
     }
 }
 
@@ -58,7 +57,7 @@ where
 
     fn into_boxed_slice_u8(self) -> Box<[u8]> {
         let key_bytes = self.key.to_boxed_bytes();
-        self.api
+        A::storage_read_api_impl()
             .storage_load_boxed_bytes(key_bytes.as_slice())
             .into_box()
     }
@@ -68,13 +67,13 @@ where
         if let Some(num) = mb.parse_as_u64() {
             num
         } else {
-            storage_get_exit(self.api, DecodeError::INPUT_TOO_LONG)
+            storage_get_exit::<A>((), DecodeError::INPUT_TOO_LONG)
         }
     }
 
     fn into_i64(self) -> i64 {
         let key_bytes = self.key.to_boxed_bytes();
-        self.api.storage_load_i64(key_bytes.as_slice())
+        A::storage_read_api_impl().storage_load_i64(key_bytes.as_slice())
     }
 
     fn into_specialized<T, F>(self, else_deser: F) -> Result<T, DecodeError>
@@ -98,33 +97,29 @@ where
     }
 }
 
-pub fn storage_get<A, T>(api: A, key: &StorageKey<A>) -> T
+pub fn storage_get<A, T>(key: ManagedRef<'_, A, StorageKey<A>>) -> T
 where
     T: TopDecode,
-    A: StorageReadApi + ManagedTypeApi + ErrorApi + Clone + 'static,
+    A: StorageReadApi + ManagedTypeApi + ErrorApi,
 {
-    T::top_decode_or_exit(
-        StorageGetInput::new(api.clone(), key),
-        api,
-        storage_get_exit,
-    )
+    T::top_decode_or_exit(StorageGetInput::new(key), (), storage_get_exit::<A>)
 }
 
 /// Useful for storage mappers.
 /// Also calls to it generated by macro.
-pub fn storage_get_len<A>(api: A, key: &StorageKey<A>) -> usize
+pub fn storage_get_len<A>(key: ManagedRef<'_, A, StorageKey<A>>) -> usize
 where
-    A: StorageReadApi + ManagedTypeApi + ErrorApi + Clone + 'static,
+    A: StorageReadApi + ManagedTypeApi + ErrorApi,
 {
-    api.storage_load_managed_buffer_len(key.buffer.get_raw_handle())
+    A::storage_read_api_impl().storage_load_managed_buffer_len(key.get_raw_handle())
 }
 
 #[inline(always)]
-fn storage_get_exit<A>(api: A, de_err: DecodeError) -> !
+fn storage_get_exit<A>(_: (), de_err: DecodeError) -> !
 where
     A: StorageReadApi + ManagedTypeApi + ErrorApi + 'static,
 {
     let mut message_buffer = ManagedBuffer::<A>::new_from_bytes(err_msg::STORAGE_DECODE_ERROR);
     message_buffer.append_bytes(de_err.message_bytes());
-    api.signal_error_from_buffer(message_buffer.get_raw_handle())
+    A::error_api_impl().signal_error_from_buffer(message_buffer.get_raw_handle())
 }
