@@ -3,7 +3,7 @@ use core::ops::{
     Mul, MulAssign, Rem, RemAssign, Shl, ShlAssign, Shr, ShrAssign, Sub, SubAssign,
 };
 
-use crate::api::ManagedTypeApi;
+use crate::api::{BigIntApi, ManagedTypeApi};
 
 use super::{BigUint, ManagedType};
 
@@ -13,8 +13,7 @@ macro_rules! binary_operator {
             type Output = BigUint<M>;
 
             fn $method(self, other: BigUint<M>) -> BigUint<M> {
-                self.type_manager()
-                    .$api_func(self.handle, self.handle, other.handle);
+                M::managed_type_impl().$api_func(self.handle, self.handle, other.handle);
                 BigUint::from_raw_handle(self.handle)
             }
         }
@@ -23,9 +22,8 @@ macro_rules! binary_operator {
             type Output = BigUint<M>;
 
             fn $method(self, other: &BigUint<M>) -> BigUint<M> {
-                let result = self.type_manager().bi_new_zero();
-                self.type_manager()
-                    .$api_func(result, self.handle, other.handle);
+                let result = M::managed_type_impl().bi_new_zero();
+                M::managed_type_impl().$api_func(result, self.handle, other.handle);
                 BigUint::from_raw_handle(result)
             }
         }
@@ -34,8 +32,7 @@ macro_rules! binary_operator {
             type Output = BigUint<M>;
 
             fn $method(self, other: &BigUint<M>) -> BigUint<M> {
-                self.type_manager()
-                    .$api_func(self.handle, self.handle, other.handle);
+                M::managed_type_impl().$api_func(self.handle, self.handle, other.handle);
                 BigUint::from_raw_handle(self.handle)
             }
         }
@@ -44,9 +41,8 @@ macro_rules! binary_operator {
             type Output = BigUint<M>;
 
             fn $method(self, other: u32) -> BigUint<M> {
-                let other_handle = self.type_manager().bi_new(other as i64);
-                self.type_manager()
-                    .$api_func(self.handle, self.handle, other_handle);
+                let other_handle = M::managed_type_impl().bi_new(other as i64);
+                M::managed_type_impl().$api_func(self.handle, self.handle, other_handle);
                 BigUint::from_raw_handle(self.handle)
             }
         }
@@ -55,10 +51,9 @@ macro_rules! binary_operator {
             type Output = BigUint<M>;
 
             fn $method(self, other: u32) -> BigUint<M> {
-                let other_handle = self.type_manager().bi_new(other as i64);
-                let result = self.type_manager().bi_new_zero();
-                self.type_manager()
-                    .$api_func(result, self.handle, other_handle);
+                let other_handle = M::managed_type_impl().bi_new(other as i64);
+                let result = M::managed_type_impl().bi_new_zero();
+                M::managed_type_impl().$api_func(result, self.handle, other_handle);
                 BigUint::from_raw_handle(result)
             }
         }
@@ -67,9 +62,8 @@ macro_rules! binary_operator {
             type Output = BigUint<M>;
 
             fn $method(self, other: u64) -> BigUint<M> {
-                let other_handle = self.type_manager().bi_new(other as i64);
-                self.type_manager()
-                    .$api_func(self.handle, self.handle, other_handle);
+                let other_handle = M::managed_type_impl().bi_new(other as i64);
+                M::managed_type_impl().$api_func(self.handle, self.handle, other_handle);
                 BigUint::from_raw_handle(self.handle)
             }
         }
@@ -78,10 +72,9 @@ macro_rules! binary_operator {
             type Output = BigUint<M>;
 
             fn $method(self, other: u64) -> BigUint<M> {
-                let other_handle = self.type_manager().bi_new(other as i64);
-                let result = self.type_manager().bi_new_zero();
-                self.type_manager()
-                    .$api_func(result, self.handle, other_handle);
+                let other_handle = M::managed_type_impl().bi_new(other as i64);
+                let result = M::managed_type_impl().bi_new_zero();
+                M::managed_type_impl().$api_func(result, self.handle, other_handle);
                 BigUint::from_raw_handle(result)
             }
         }
@@ -102,32 +95,28 @@ macro_rules! binary_assign_operator {
         impl<M: ManagedTypeApi> $trait<BigUint<M>> for BigUint<M> {
             #[inline]
             fn $method(&mut self, other: Self) {
-                self.type_manager()
-                    .$api_func(self.handle, self.handle, other.handle);
+                M::managed_type_impl().$api_func(self.handle, self.handle, other.handle);
             }
         }
 
         impl<M: ManagedTypeApi> $trait<&BigUint<M>> for BigUint<M> {
             #[inline]
             fn $method(&mut self, other: &BigUint<M>) {
-                self.type_manager()
-                    .$api_func(self.handle, self.handle, other.handle);
+                M::managed_type_impl().$api_func(self.handle, self.handle, other.handle);
             }
         }
 
         impl<M: ManagedTypeApi> $trait<u32> for BigUint<M> {
             fn $method(&mut self, other: u32) {
-                let other_handle = self.type_manager().bi_new(other as i64);
-                self.type_manager()
-                    .$api_func(self.handle, self.handle, other_handle);
+                let other_handle = M::managed_type_impl().bi_new(other as i64);
+                M::managed_type_impl().$api_func(self.handle, self.handle, other_handle);
             }
         }
 
         impl<M: ManagedTypeApi> $trait<u64> for BigUint<M> {
             fn $method(&mut self, other: u64) {
-                let other_handle = self.type_manager().bi_new(other as i64);
-                self.type_manager()
-                    .$api_func(self.handle, self.handle, other_handle);
+                let other_handle = M::managed_type_impl().bi_new(other as i64);
+                M::managed_type_impl().$api_func(self.handle, self.handle, other_handle);
             }
         }
     };
@@ -149,7 +138,7 @@ macro_rules! shift_traits {
 
             #[inline]
             fn $method(self, rhs: usize) -> BigUint<M> {
-                self.type_manager().$api_func(self.handle, self.handle, rhs);
+                M::managed_type_impl().$api_func(self.handle, self.handle, rhs);
                 self
             }
         }
@@ -158,8 +147,8 @@ macro_rules! shift_traits {
             type Output = BigUint<M>;
 
             fn $method(self, rhs: usize) -> BigUint<M> {
-                let result = self.type_manager().bi_new_zero();
-                self.type_manager().$api_func(result, self.handle, rhs);
+                let result = M::managed_type_impl().bi_new_zero();
+                M::managed_type_impl().$api_func(result, self.handle, rhs);
                 BigUint::from_raw_handle(result)
             }
         }
@@ -174,7 +163,7 @@ macro_rules! shift_assign_traits {
         impl<M: ManagedTypeApi> $shift_assign_trait<usize> for BigUint<M> {
             #[inline]
             fn $method(&mut self, rhs: usize) {
-                self.type_manager().$api_func(self.handle, self.handle, rhs);
+                M::managed_type_impl().$api_func(self.handle, self.handle, rhs);
             }
         }
     };
